from awaits.common_data import CommonData


class config(object):
    """
    Установка глобальных параметров.
    """
    allowed_settings = {
        'pool_size': (int, ),
        'delay': (int, float, ),
    }

    @staticmethod
    def set(**kwargs):
        new_kwargs = {}
        for key, value in kwargs.items():
            if key not in config.allowed_settings.keys():
                raise ValueError(f'"{key}" variable is not allowed for the awaits settings.')
            allowed_types = config.allowed_settings.get(key)
            is_allowed = False
            for one in allowed_types:
                if isinstance(value, one):
                    is_allowed = True
            if not is_allowed:
                allowed_types = ', '.join(allowed_types)
                raise TypeError(f'Variable "{key}" has not one of types: {allowed_types}.')
            new_kwargs[key] = value
        CommonData(**new_kwargs)



class CommonData:
    pool_size = 10
    delay = 0.01

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self.__class__, key, value)

    def __new__(cls, **kwargs):
        if not hasattr(cls, 'instance'):
            cls.instance = super(CommonData, cls).__new__(cls)
        return cls.instance



from awaits.pools.threads_pool import ThreadsPool


class ThreadsPoolsRoom:
    def __init__(self, pool_size):
        self.pool_size = pool_size
        self.pools = {}

    def __getitem__(self, key):
        if not isinstance(key, str):
            raise KeyError(f'Key of the pools room must be a string, not {key.__class__.__name__}.')
        if key not in self.pools:
            self.pools[key] = ThreadsPool(self.pool_size)
        return self.pools[key]

    def __len__(self):
        return len(self.pools)



from functools import wraps
from asyncio import sleep

from awaits.common_data import CommonData
from awaits.utils.get_pool_for_decorator import get_pool_for_decorator
from awaits.utils.end_of_wrappers import end_of_wrappers


def awaitable(*args, pool=None, delay=None):
    pool = get_pool_for_decorator(pool)

    def wrapper_of_wrapper(function):
        @wraps(function)
        async def wrapper(*args, **kwargs):
            sleep_time = CommonData().delay if delay is None else delay
            task = pool.do(function, *args, **kwargs)
            while not task.done:
                await sleep(sleep_time)
            if task.error:
                raise task.exception
            return task.result
        return wrapper

    return end_of_wrappers(args, wrapper_of_wrapper)



from functools import wraps
from typing import Union, Optional

from awaits.utils.get_pool_for_decorator import get_pool_for_decorator
from awaits.utils.end_of_wrappers import end_of_wrappers
from awaits.pools.abstract_pool import AbstractPool


def shoot(*args, pool: Optional[Union[str, AbstractPool]] = None):
    pool = get_pool_for_decorator(pool)

    def wrapper_of_wrapper(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            task = pool.do(func, *args, **kwargs)
            return task
        return wrapper
    
    return end_of_wrappers(args, wrapper_of_wrapper)
